package com.bcxin.sp.config.aspect;

import com.alibaba.fastjson.JSON;
import com.bcxin.sp.config.annotation.OperLog;
import com.bcxin.sp.work.dao.activity.base.ActivityMapper;
import com.bcxin.sp.work.entity.domain.activity.base.Activity;
import com.bcxin.sp.work.entity.domain.log.OperateLog;
import com.bcxin.sp.work.entity.domain.user.User;
import com.bcxin.sp.work.entity.dto.AjaxResult;
import com.bcxin.sp.work.util.Const.DictConst;
import com.bcxin.sp.work.util.Const.GlobalConst;
import com.bcxin.sp.work.util.HttpContextUtils;
import com.bcxin.sp.work.util.OperateLogUtil;
import com.bcxin.sp.work.util.StringUtil;
import com.bcxin.sp.work.util.WebUtil;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.CodeSignature;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayOutputStream;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.*;

/**
 * 操作日志切面
 */
@Aspect
@Component
public class OperateLogAspect {

    @Autowired
    ActivityMapper activityMapper;

    @Pointcut("@annotation(com.bcxin.sp.config.annotation.OperLog)")
    public void logPointCut() {
    }

    @Around("logPointCut()")
    public Object around(ProceedingJoinPoint point) {
        long beginTime = System.currentTimeMillis();
        OperateLog operateLog = null;
        Object result = null;
        //保存日志
        HttpServletRequest request = HttpContextUtils.getHttpServletRequest();
        Object userObj = request.getSession().getAttribute(GlobalConst.LOGIN_USER);
        Map<String, Object> paramMap = null;
        try {
            //调用目标方法
            result = point.proceed();

            // 如果是登录操作，userObj为空，需要重新获取
            if(userObj==null){
                userObj = request.getSession().getAttribute(GlobalConst.LOGIN_USER);
            }
            // 生成操作日志
            operateLog = createLog(point,userObj, request, beginTime, true);


            // 是否ajax请求
            if (WebUtil.isAjaxRequest(request)) {
                if (result instanceof AjaxResult) {
                    AjaxResult ajaxResult = (AjaxResult) result;
                    if (ajaxResult.isSuccessful()) {
                        operateLog.setResult(1);
                    } else {
                        operateLog.setResult(0);
                        operateLog.setException(ajaxResult.getMsg());
                    }
                }
            }
        } catch (Throwable e) {
            e.printStackTrace();
            //保存日志
            operateLog = createLog(point, userObj,beginTime, false);
            if (operateLog != null) {
                operateLog.setException(getStackMessage(e));
            }
        }

        if (operateLog != null) {
            OperateLogUtil.saveOperate(operateLog);
        }
        return result;
    }


    public OperateLog createLog(ProceedingJoinPoint point, Object userObj,Long startTime, boolean result) {
        HttpServletRequest request = HttpContextUtils.getHttpServletRequest();
        return createLog(point, userObj,request, startTime, result);
    }

    /**
     * 生成操作日志
     *
     * @param point
     * @param startTime
     * @param result
     * @return
     */
    public OperateLog createLog(ProceedingJoinPoint point, Object userObj,HttpServletRequest request, Long startTime, boolean result) {
        MethodSignature signature = (MethodSignature) point.getSignature();
        Method method = signature.getMethod();
        OperLog operLog = method.getAnnotation(OperLog.class);
        if (operLog == null) {
            return null;
        }

        OperateLog log = new OperateLog();
        Map<String,Object> paramMap = getParams(point);
        if (paramMap != null) {
            log.setParams(JSON.toJSONString(paramMap));
        }
        log.setAddTime(new Date());
        log.setMenuName(operLog.menuName());
        log.setOperateType(transferOperateType(paramMap,operLog.operateType(),operLog.operateTypeMethod()));
        log.setOperateObj(transferOperateObj(paramMap, operLog.operateObj()));
        log.setRequestMethod(request.getMethod());
        log.setRequestUrl(request.getRequestURI());
        log.setIp(WebUtil.getIP(request));
        long endTime = System.currentTimeMillis();
        log.setStartTime(startTime);
        log.setEndTime(endTime);
        log.setSpeedTime(endTime - startTime);

        //用户名
        if(request.getRequestURI().contains("/business/logout")){
            userObj=paramMap.get("user");
        }
        if (userObj != null) {
            User user = (User) userObj;
            log.setUserId(user.getId());
            log.setUserName(user.getRealName());
            log.setProvince(user.getProvince());
            log.setCity(user.getCity());
            log.setArea(user.getArea());
            log.setProvinceName(user.getProvincename());
            log.setCityName(user.getCityname());
            log.setAreaName(user.getAreaname());
            if(request.getRequestURI().contains("/business/logout")
                    || request.getRequestURI().contains("/doLogin")){
                log.setOperateObj(user.getRealName()+"("+user.getUsername()+")");
            }
        }
        // 是否成功
        if (result) {
            log.setResult(1);
        } else {
            log.setResult(0);
        }
        return log;
    }

    /**
     * 转换操作类型
     * @param map
     * @param operateType
     * @return
     */
    public String transferOperateType(Map<String, Object> map, String operateType,String methodName){
        String val=operateType;
        try {
            if (operateType.startsWith("#")) {
                operateType = operateType.substring(1);
                String[] tmp = operateType.split("\\.");
                if (tmp.length == 1) {
                    val= map.containsKey(tmp[0]) ? map.get(tmp[0]).toString() : operateType;
                } else if (tmp.length == 2) {
                    Object obj = map.get(tmp[0]);
                    if (obj != null) {
                        Map<String, Object> objMap = objectToMap(obj);
                        if (objMap != null) {
                            val= objMap.containsKey(tmp[1]) ? objMap.get(tmp[1]).toString() : operateType;
                        }
                    }
                }
            }

            // 如果有转换的方法，需要执行转换方法
            if(StringUtil.isNotEmpty(methodName)){
                if(Objects.equals(methodName,"transferCS")){
                    val= transferCS(val);
                }else if(Objects.equals(methodName,"transferSL")){
                    val=transferSL(val);
                }else if(Objects.equals(methodName,"transferApprove")){
                    val=transferApprove(val);
                }else if(Objects.equals(methodName,"transferRole")){
                    val=transferRole(val);
                }else if(Objects.equals(methodName,"transferWorkFlow")){
                    val=transferWorkFlow(val);
                }else if(Objects.equals(methodName,"transferUser")){
                    val=transferUser(val);
                }
            }
        }catch (Exception e){
            e.printStackTrace();
        }
        return val;
    }

    /**
     * 转换操作对象
     * @param map
     * @param operateObj
     * @return
     */
    public String transferOperateObj(Map<String, Object> map, String operateObj) {
        try {
            if (operateObj.startsWith("#")) {
                operateObj = operateObj.substring(1);
                String[] tmp = operateObj.split("\\.");
                if (tmp.length == 1) {
                    if(Objects.equals("activityId",tmp[0]) || Objects.equals("sl_activityId",tmp[0])){
                        return getActivityName(map.containsKey(tmp[0]) ? map.get(tmp[0]).toString() : operateObj);
                    }
                    return map.containsKey(tmp[0]) ? map.get(tmp[0]).toString() : operateObj;
                } else if (tmp.length == 2) {
                    Object obj = map.get(tmp[0]);
                    if (obj != null) {
                        Map<String, Object> objMap = objectToMap(obj);
                        if(objMap!=null){
                            if(Objects.equals("activityId",tmp[1]) || Objects.equals("sl_activityId",tmp[1])){
                                return getActivityName(map.containsKey(tmp[1]) ? map.get(tmp[1]).toString() : operateObj);
                            }
                            Object val = objMap.get(tmp[1]);
                            return objMap.containsKey(tmp[1]) ? val==null?"":val+"" : operateObj;
                        }else{
                            return "";
                        }
                    }
                }
            }
            return map.containsKey(operateObj) ? map.get(operateObj).toString() : operateObj;
        }catch (Exception e){
            e.printStackTrace();
            return "";
        }
    }

    /**
     * 返回活动名称
     */
    public String getActivityName(String activityId){
        Long id=null;
        if(StringUtil.isNotEmpty(activityId)){
            try{
                id=Long.parseLong(activityId);
            }catch (Exception e){

            }
        }
        if(id!=null) {
            Activity activity = activityMapper.findFullById(id);
            String activityName=activity.getFullName();
            return activityName;
        }
        return activityId;
    }

    /**
     * 获取参数
     * @param joinPoint
     * @return
     */
    public Map<String, Object> getParams(ProceedingJoinPoint joinPoint) {
        try {
            Object[] args = joinPoint.getArgs();
            String[] paramNames = ((CodeSignature) joinPoint.getSignature()).getParameterNames();

            Map<String, Object> map = new HashMap<>();
            for (int i = 0; i < paramNames.length; i++) {
                if (args[i] instanceof HttpServletRequest) {
                    HttpServletRequest req=(HttpServletRequest)args[i];
                    Enumeration<String> reqParams= req.getParameterNames();
                    while (reqParams.hasMoreElements()){
                        String key=reqParams.nextElement();
                        map.put(key, req.getParameter(key));
                    }
                    continue;
                }
                map.put(paramNames[i], args[i]);
            }
            return map;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 获取异常堆栈信息
     * @param e
     * @return
     */
    public String getStackMessage(Throwable e) {
        ByteArrayOutputStream buf = new ByteArrayOutputStream();
        try {
            e.printStackTrace(new java.io.PrintWriter(buf, true));
            String expMessage = buf.toString();
            return expMessage;
        } catch (Exception ex) {
            return "";
        } finally {
            try {
                buf.close();
            } catch (Exception ex1) {

            }
        }
    }

    /**
     * 对象转map
     * @param obj
     * @return
     * @throws IllegalAccessException
     */
    public Map<String, Object> objectToMap(Object obj) throws IllegalAccessException {
        Map<String, Object> map = new HashMap<>();
        Class<?> clazz = obj.getClass();
        for (Field field : clazz.getDeclaredFields()) {
            field.setAccessible(true);
            String fieldName = field.getName();
            Object value = field.get(obj);
            map.put(fieldName, value);
        }
        return map;
    }

    /**
     * 转换初审
     * @param value
     * @return
     */
    public String transferCS(String value){
        if(Objects.equals(value, DictConst.STATUS_REJECT)){
            return "初审退回";
        }else if(Objects.equals(value,DictConst.STATUS_YES)){
            return "初审通过";
        }
        return value;
    }

    /**
     * 转换受理
     * @param value
     * @return
     */
    public String transferSL(String value){
        if(Objects.equals(value,DictConst.STATUS_BSL)){
            return "不受理";
        }else if(Objects.equals(value,DictConst.STATUS_SL)){
            return "受理通过";
        }
        return value;
    }

    /**
     * 转换审批
     * @param value
     * @return
     */
    public String transferApprove(String value){
        if(Objects.equals(value,DictConst.STATUS_REJECT)){
            return "审批退回";
        }
        if(Objects.equals(value,DictConst.STATUS_NO)){
            return "审批不许可";
        }
        if(Objects.equals(value,DictConst.STATUS_YES)){
            return "审批许可";
        }
        return value;
    }

    public String transferRole(String value){
        if(StringUtil.isNotEmpty(value)){
            return "修改审批角色";
        }else{
            return "添加审批角色";
        }
    }

    public String transferWorkFlow(String value){
        if(StringUtil.isEmpty(value)){
            return "添加流程";
        }else{
            return "修改流程";
        }
    }

    public String transferUser(String value){
        if(StringUtil.isEmpty(value)){
            return "添加用户";
        }else{
            return "修改用户";
        }
    }

}
