package com.bcxin.sp.config.interceptor;

import com.bcxin.sp.config.filter.html.EscapeUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;
import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * 防止sql注入,xss攻击
 *
 * @author ZXF
 * @create 2022-07-28 15:55
 */
public class XssHttpServletRequestWrapper extends HttpServletRequestWrapper {
    private static Logger logger = LoggerFactory.getLogger(XssHttpServletRequestWrapper.class);
    private static String key = "and|exec|insert|select|delete|update|count|*|%|chr|mid|master|truncate|char|declare|;|or|-|+";
    private static Set<String> notAllowedKeyWords = new HashSet<String>(0);
    private static String replacedString = "INVALID";

    static {
        String keyStr[] = key.split("\\|");
        for (String str : keyStr) {
            notAllowedKeyWords.add(str);
        }
    }

    public XssHttpServletRequestWrapper(HttpServletRequest request) throws IOException {
        super(request);
    }

    /**
     * @Description 覆盖getParameter方法，将参数和参数值做xss过滤
     * @Date 2020/5/20 9:57
     */
    @Override
    public String getParameter(String name) {
        String value = super.getParameter(name);
        if(StringUtils.isEmpty(value)){
            return null;
        }
        return cleanXss(value);
    }

    @Override
    public Map<String, String[]> getParameterMap() {
        Map<String,String[]> values = super.getParameterMap();
        if(null == values){
            return null;
        }
        Map<String,String[]> result = new HashMap<>();
        for (String key : values.keySet()) {
            String encodedKey = cleanXss(key);
            int count = values.get(key).length;
            String[] encodedValues = new String[count];
            for(int i = 0;i < count;i++){
                encodedValues[i] = cleanXss(values.get(key)[i]);
            }
            result.put(encodedKey,encodedValues);
        }
        return result;
    }

    @Override
    public String getHeader(String name) {
        String value = super.getHeader(name);
        if(StringUtils.isEmpty(value)){
            return null;
        }
//        return cleanXss(value);
        return value;
    }

    @Override
    public String[] getParameterValues(String name) {
        String[] values = super.getParameterValues(name);
        if (values != null){
            int length = values.length;
            String[] escapseValues = new String[length];
            for (int i = 0; i < length; i++){
                // 防xss攻击和过滤前后空格
                escapseValues[i] = cleanXss(values[i]);
            }
            return escapseValues;
        }
        return super.getParameterValues(name);
    }


    /**
     * @Description 解析参数
     * @Date 2020/5/20 10:01
     */
    private String cleanXss(String valueP){
        String value = EscapeUtil.clean(valueP).trim();
        value = cleanSqlKeyWords(value);
        return value;
    }

    /**
     * @Description 解析参数SQL关键字
     * @Date 2020/5/20 10:01
     */
    private String cleanSqlKeyWords(String value){
        if(value.contains("Chrome/") || value.contains("application/json, text/javascript, */*;")){
            return value;
        }
        String paramValue = value.toLowerCase();
        int sign=0;
        for (String keyword : notAllowedKeyWords) {
            if(paramValue.length() > keyword.length() + 4
                    && (paramValue.contains(" " + keyword)
                    || paramValue.contains(keyword + " ")
                    || paramValue.contains(" " + keyword + " ")
                    || paramValue.contains("%20" + keyword)
                    || paramValue.contains(keyword + "%20")
                    || paramValue.contains("%20" + keyword + "%20"))){
                paramValue = StringUtils.replace(paramValue,keyword,replacedString);
                sign=1;
                logger.error( "链接已被过滤，因为参数中包含不允许sql的关键词(" + keyword + ");参数：" + value + "；过滤后的参数：" + paramValue);
            }
        }
        if(value.toLowerCase().equals(paramValue)){
            return value;
        }
        if(sign==1){
            paramValue = value;
        }
        return paramValue;
    }

    public boolean checkSqlKeyWords(String value){
        String paramValue = value;
        for (String keyword : notAllowedKeyWords) {
            if (paramValue.length() > keyword.length() + 4
                    && (paramValue.contains(" "+keyword)
                    || paramValue.contains(keyword+" ")
                    || paramValue.contains(" "+keyword+" ")
                    || paramValue.contains("%20" + keyword)
                    || paramValue.contains(keyword + "%20")
                    || paramValue.contains("%20" + keyword + "%20"))
            ) {
                logger.error(this.getRequestURI()+ "参数中包含不允许sql的关键词(" + keyword
                        + ")");
                return true;
            }
        }
        return false;
    }
}
